@@ -329,8 +329,9 @@ Either min/max or range can be given but not both. min/max are numerical values.
 '*range' is a string 'min:max' with either one allowed to be omitted; it can
 also be a [min,max] tuple or list. '*inv' is a boolean that reverses this axis.
 If the bounds are known, this can also be accomplished by setting max < min.
+Passing in both max < min AND inv also results in a reversed axis.
 
-If no nformation about a range is given, it is not touched: the previous zoom
+If no information about a range is given, it is not touched: the previous zoom
 settings are preserved.
 
 The y2 axis is the secondary y-axis that is enabled by the 'y2' curve option.
@@ -432,7 +433,7 @@ of writing to a gnuplot process. Useful to see what commands would be sent to
 gnuplot. This is a dry run. Note that this dump will contain binary data unless
 ascii-only plotting is enabled (see below). This is also useful to generate
 gnuplot scripts since the dumped output can be sent to gnuplot later, manually
-if desired.
+if desired. Look at the 'notest' option for a less verbose dump.
 
 - log
 
@@ -450,6 +451,11 @@ for most plots, but not for all of them. An example where binary plotting
 doesn't work is 'with labels', and this option exists to force ASCII
 communication
 
+- notest
+
+Don't check for failure after each gnuplot command. And don't test all the plot
+options before creating the plot. This is generally only useful for debugging or
+for more sparse 'dump' functionality.
 
 ** Curve options
 
@@ -665,6 +671,7 @@ one can do
 
 This command is equivalent to the 'hardcopy' shorthand used previously, but the
 fonts and sizes can be changed.
+
 '''
 
 
@@ -677,12 +684,15 @@ import sys
 import os
 import re
 import select
+import numbers
 import numpy as np
 import numpysane as nps
 
 
 # note that 'with' is both a known plot and curve option
-knownPlotOptions = frozenset(('3d', 'dump', 'ascii', 'log',
+knownPlotOptions = frozenset(('dump', 'ascii', 'log', 'notest',
+
+                              '3d',
                               'cmds', 'set', 'unset', 'square', 'square_xy', 'title',
                               'hardcopy', 'terminal', 'output',
                               'with', 'equation',
@@ -853,49 +863,48 @@ class gnuplotlib:
                     cmds.append(setunset + ' ' + self.plotOptions[setunset])
 
         # set the plot bounds
-
         for axis in ('x', 'y', 'y2', 'z', 'cb'):
 
-            # if we have min AND max AND inv, I make sure that min>max, and
-            # suppress the inv. This is because in gnuplot 'plot [min:max]
-            # reverse' ingores the 'reverse' if both min and max are given
-            if axis + 'min' in self.plotOptions and \
-               axis + 'max' in self.plotOptions and \
-               self.plotOptions.get(axis + 'inv'):
-                minval = min(self.plotOptions[axis + 'min'], self.plotOptions[axis + 'max'])
-                maxval = max(self.plotOptions[axis + 'min'], self.plotOptions[axis + 'max'])
-                self.plotOptions[axis + 'min'] = maxval
-                self.plotOptions[axis + 'max'] = minval
-                self.plotOptions[axis + 'inv'] = False
+            # I deal with range bounds here. These can be given for the various
+            # axes by variables (W-axis here; replace W with x, y, z, etc):
+            #
+            #   Wmin, Wmax, Winv, Wrange
+            #
+            # Wrange is mutually exclusive with Wmin and Wmax. Winv turns
+            # reverses the direction of the axis. This can also be achieved by
+            # passing in Wmin>Wmax or Wrange[0]>Wrange[1]. If this is done then
+            # Winv has no effect, i.e. setting Wmin>Wmax AND Winv results in a
+            # flipped axis.
+            opt_min   = self.plotOptions.get( axis + 'min'   )
+            opt_max   = self.plotOptions.get( axis + 'max'   )
+            opt_range = self.plotOptions.get( axis + 'range' )
+            opt_inv   = self.plotOptions.get( axis + 'inv'   )
 
             # If a bound isn't given I want to set it to the empty string, so I can communicate it simply
             # to gnuplot
-            rangeopt_name = axis + 'range'
-            for minmax in ('min', 'max'):
-                opt = axis + minmax
-                if not opt in self.plotOptions:
-                    self.plotOptions[opt] = ''
-                else:
-                    if rangeopt_name in self.plotOptions:
-                        raise GnuplotlibError("Both {} and {} not allowed at the same time".format(opt,rangeopt_name))
-                    self.plotOptions[opt] = str(self.plotOptions[opt])
+            if (opt_min is not None or opt_max is not None) and opt_range is not None:
+                raise GnuplotlibError("{0}min/{0}max and {0}range are mutually exclusive".format(axis))
 
-            # if any of the ranges are given, set the range
-            if len(self.plotOptions[axis + 'min'] + self.plotOptions[axis + 'max']):
-                rangeopt_val = ':'.join((self.plotOptions[axis + 'min'], self.plotOptions[axis + 'max']))
-            elif rangeopt_name in self.plotOptions:
-                # A range was given. If it's a string, just take it. It can also
-                # be a two-value list for the min/max
-                rangeopt_val = self.plotOptions[rangeopt_name]
-                if isinstance(rangeopt_val, (list, tuple)):
-                    rangeopt_val = ':'.join(str(x) for x in rangeopt_val)
-            else:
-                rangeopt_val = None
+            # if we have a range, copy it to min/max and just work with those
+            if opt_range is not None:
+                if not isinstance(opt_range, (list, tuple)):
+                    opt_range = [ None if x == '*' else float(x) for x in opt_range.split(':')]
+                if len(opt_range) != 2:
+                    raise GnuplotlibError('{}range should have exactly 2 elements'.format(axis))
+                opt_min,opt_max = opt_range
+                opt_range = None
 
-            if rangeopt_val is not None:
-                cmds.append( "set {} [{}] {}".format(rangeopt_name,
-                                                     rangeopt_val,
-                                                     'reverse' if self.plotOptions.get(axis + 'inv') else ''))
+            # apply the axis inversion. It's only needed if we're given both
+            # bounds and they aren't flipped
+            if opt_inv:
+                if opt_min is not None and opt_max is not None and opt_min < opt_max:
+                    opt_min,opt_max = opt_max,opt_min
+
+            cmds.append( "set {}range [{}:{}] {}reverse".
+                         format(axis,
+                                '*' if opt_min is None else opt_min,
+                                '*' if opt_max is None else opt_max,
+                                '' if opt_inv else 'no'))
 
             # set the curve labels
             if not axis == 'cb':
@@ -1050,6 +1059,9 @@ defaults are acceptable, use 'hardcopy' only, otherwise use 'terminal' and
     # I test the plot command by making a dummy plot with the test command.
     def _testPlotcmd(self, cmd, data):
 
+        if self.plotOptions.get('notest'):
+            return
+
         # I don't actually want to see the plot, I just want to make sure that
         # no errors are thrown. I thus send the output to /dev/null. Note that I
         # never actually read stdout, so if this test plot goes to the default
@@ -1075,6 +1087,9 @@ defaults are acceptable, use 'hardcopy' only, otherwise use 'terminal' and
                 barfmsg += "Warnings:\n" + "\n".join(warnings)
             raise GnuplotlibError(barfmsg)
 
+        # select the default terminal in case that's what we want
+        self._safelyWriteToPipe("set terminal pop; set terminal push", 'terminal')
+
 
     # syncronizes the child and parent processes. After _checkpoint() returns, I
     # know that I've read all the data from the child. Extra data that represents
@@ -1086,6 +1101,9 @@ defaults are acceptable, use 'hardcopy' only, otherwise use 'terminal' and
         # yet arrived. I thus print out a checkpoint message and keep reading the
         # child's STDERR pipe until I get that message back. Any errors would have
         # been printed before this
+        if self.plotOptions.get('notest'):
+            return None, None
+
         checkpoint = "gpsync{}xxx".format(self.sync_count)
         self.sync_count += 1
 
@@ -1369,11 +1387,25 @@ and/or gnuplot itself. Please report this as a gnuplotlib bug''')
 
     def _massageAndValidateArgs(self, curves, curveOptions_base):
 
-        # Collect all the passed data into a tuple of lists, one curve per list
+        # Collect all the passed data into a tuple of lists, one curve per list.
+        # The input is either a bunch of numerical arrays, in which we have one
+        # curve (ignoring broadcasting) or a bunch of tuples containing
+        # numerical arrays, where each tuple represents a curve.
+        #
+        # These numerical arrays can be numpy arrays or scalars. If we see
+        # scalars, we convert them to a numpy array so that everything
+        # downstream can assume we have arrays
+
+        # convert any scalars in the data list
+        curves = [ np.array((c,)) if isinstance(c, numbers.Real) else c for c in curves ]
         if all(type(curve) is np.ndarray for curve in curves):
             curves = (list(curves),)
         elif all(type(curve) is tuple for curve in curves):
-            curves = [ list(curve) for curve in curves ]
+            # we have a list of tuples. I convert this into a list of lists, and
+            # each scalar in each list becomes a numpy array
+            curves = [ [ np.array((c,)) if isinstance(c, numbers.Real) else c
+                         for c in curve ]
+                       for curve in curves ]
         else:
             raise GnuplotlibError("all data arguments should be of type ndarray (one curve) or tuples")
 
@@ -1382,7 +1414,9 @@ and/or gnuplot itself. Please report this as a gnuplotlib bug''')
         for curve in curves:
             if not type(curve[-1]) is dict:
                 curve.append({})
-            curve[-1].update(curveOptions_base)
+            for k in curveOptions_base:
+                if k not in curve[-1]:
+                    curve[-1][k] = curveOptions_base[k]
 
         # I convert the curve definition from a list of
         #    (data, data, data, ..., {options})
@@ -1522,10 +1556,6 @@ and/or gnuplot itself. Please report this as a gnuplotlib bug''')
         self._testPlotcmd(testcmd, testdata)
 
         # tests ok. Now set the terminal and actually make the plot!
-
-        # select the default terminal in case that's what we want
-        self._safelyWriteToPipe("set terminal pop; set terminal push", 'terminal')
-
         if 'terminal' in self.plotOptions:
             self._safelyWriteToPipe("set terminal " + self.plotOptions['terminal'],
                                     'terminal')
